@@ -3,6 +3,7 @@ import select
 import outcome
 from contextlib import contextmanager
 import attr
+import errno
 
 from .. import _core
 from ._run import _public
@@ -120,7 +121,26 @@ class KqueueIOManager:
 
         def abort(_):
             event = select.kevent(fd, filter, select.KQ_EV_DELETE)
-            self._kqueue.control([event], 0)
+            try:
+                self._kqueue.control([event], 0)
+            except OSError as exc:
+                # kqueue tracks individual fds (*not* the underlying file
+                # object, see _io_epoll.py for a long discussion of why this
+                # distinction matters), and automatically deregisters an event
+                # if the fd is closed. So if kqueue.control says that it
+                # doesn't know about this event, then probably it's because
+                # the fd was closed behind our backs. (Too bad we can't ask it
+                # to wake us up when this happens, versus discovering it after
+                # the fact... oh well, you can't have everything.)
+                #
+                # FreeBSD reports this using EBADF. macOS uses ENOENT.
+                if exc.errno in (
+                    errno.EBADF, errno.ENOENT
+                ):  # pragma: no branch
+                    pass
+                else:  # pragma: no cover
+                    # As far as we know, this branch can't happen.
+                    raise
             return _core.Abort.SUCCEEDED
 
         await self.wait_kevent(fd, filter, abort)
