@@ -17,7 +17,6 @@ from time import perf_counter
 from sniffio import current_async_library_cvar
 
 import attr
-from async_generator import isasyncgen
 from sortedcontainers import SortedDict
 from outcome import Error, Value, capture
 
@@ -36,7 +35,7 @@ from ._traps import (
 )
 from .. import _core
 from .._deprecate import deprecated
-from .._util import Final, NoPublicConstructor
+from .._util import Final, NoPublicConstructor, coroutine_or_error
 
 _NO_SEND = object()
 
@@ -60,7 +59,7 @@ INSTRUMENT_LOGGER = logging.getLogger("trio.abc.Instrument")
 
 
 # On 3.7+, Context.run() is implemented in C and doesn't show up in
-# tracebacks. On 3.6 and earlier, we use the contextvars backport, which is
+# tracebacks. On 3.6, we use the contextvars backport, which is
 # currently implemented in Python and adds 1 frame to tracebacks. So this
 # function is a super-overkill version of "0 if sys.version_info >= (3, 7)
 # else 1". But if Context.run ever changes, we'll be ready!
@@ -790,13 +789,13 @@ class Nursery(metaclass=NoPublicConstructor):
 
     @property
     def child_tasks(self):
-        """(`frozenset`): Contains all the child :class:`~trio.hazmat.Task`
+        """(`frozenset`): Contains all the child :class:`~trio.lowlevel.Task`
         objects which are still running."""
         return frozenset(self._children)
 
     @property
     def parent_task(self):
-        "(`~trio.hazmat.Task`):  The Task that opened this nursery."
+        "(`~trio.lowlevel.Task`):  The Task that opened this nursery."
         return self._parent_task
 
     def _add_exc(self, exc):
@@ -929,8 +928,8 @@ class Nursery(metaclass=NoPublicConstructor):
         :meth:`start` is cancelled, then the child task is also
         cancelled.
 
-        When the child calls ``task_status.started()``, it's moved from
-        out from underneath :meth:`start` and into the given nursery.
+        When the child calls ``task_status.started()``, it's moved out
+        from underneath :meth:`start` and into the given nursery.
 
         If the child task passes a value to
         ``task_status.started(value)``, then :meth:`start` returns this
@@ -968,7 +967,7 @@ class Nursery(metaclass=NoPublicConstructor):
 
 
 @attr.s(eq=False, hash=False, repr=False)
-class Task:
+class Task(metaclass=NoPublicConstructor):
     _parent_nursery = attr.ib()
     coro = attr.ib()
     _runner = attr.ib()
@@ -1144,7 +1143,7 @@ class Runner:
           :data:`~math.inf` if there are no pending deadlines.
         * ``run_sync_soon_queue_size`` (int): The number of
           unprocessed callbacks queued via
-          :meth:`trio.hazmat.TrioToken.run_sync_soon`.
+          :meth:`trio.lowlevel.TrioToken.run_sync_soon`.
         * ``io_statistics`` (object): Some statistics from Trio's I/O
           backend. This always has an attribute ``backend`` which is a string
           naming which operating-system-specific I/O backend is in use; the
@@ -1210,7 +1209,7 @@ class Runner:
         to calling :func:`reschedule` once.)
 
         Args:
-          task (trio.hazmat.Task): the task to be rescheduled. Must be blocked
+          task (trio.lowlevel.Task): the task to be rescheduled. Must be blocked
               in a call to :func:`wait_task_rescheduled`.
           next_send (outcome.Outcome): the value (or error) to return (or
               raise) from :func:`wait_task_rescheduled`.
@@ -1247,89 +1246,7 @@ class Runner:
         # Call the function and get the coroutine object, while giving helpful
         # errors for common mistakes.
         ######
-
-        def _return_value_looks_like_wrong_library(value):
-            # Returned by legacy @asyncio.coroutine functions, which includes
-            # a surprising proportion of asyncio builtins.
-            if isinstance(value, collections.abc.Generator):
-                return True
-            # The protocol for detecting an asyncio Future-like object
-            if getattr(value, "_asyncio_future_blocking", None) is not None:
-                return True
-            # asyncio.Future doesn't have _asyncio_future_blocking until
-            # 3.5.3. We don't want to import asyncio, but this janky check
-            # should work well enough for our purposes. And it also catches
-            # tornado Futures and twisted Deferreds. By the time we're calling
-            # this function, we already know something has gone wrong, so a
-            # heuristic is pretty safe.
-            if value.__class__.__name__ in ("Future", "Deferred"):
-                return True
-            return False
-
-        try:
-            coro = async_fn(*args)
-        except TypeError:
-            # Give good error for: nursery.start_soon(trio.sleep(1))
-            if isinstance(async_fn, collections.abc.Coroutine):
-                raise TypeError(
-                    "Trio was expecting an async function, but instead it got "
-                    "a coroutine object {async_fn!r}\n"
-                    "\n"
-                    "Probably you did something like:\n"
-                    "\n"
-                    "  trio.run({async_fn.__name__}(...))            # incorrect!\n"
-                    "  nursery.start_soon({async_fn.__name__}(...))  # incorrect!\n"
-                    "\n"
-                    "Instead, you want (notice the parentheses!):\n"
-                    "\n"
-                    "  trio.run({async_fn.__name__}, ...)            # correct!\n"
-                    "  nursery.start_soon({async_fn.__name__}, ...)  # correct!"
-                    .format(async_fn=async_fn)
-                ) from None
-
-            # Give good error for: nursery.start_soon(future)
-            if _return_value_looks_like_wrong_library(async_fn):
-                raise TypeError(
-                    "Trio was expecting an async function, but instead it got "
-                    "{!r} – are you trying to use a library written for "
-                    "asyncio/twisted/tornado or similar? That won't work "
-                    "without some sort of compatibility shim."
-                    .format(async_fn)
-                ) from None
-
-            raise
-
-        # We can't check iscoroutinefunction(async_fn), because that will fail
-        # for things like functools.partial objects wrapping an async
-        # function. So we have to just call it and then check whether the
-        # return value is a coroutine object.
-        if not isinstance(coro, collections.abc.Coroutine):
-            # Give good error for: nursery.start_soon(func_returning_future)
-            if _return_value_looks_like_wrong_library(coro):
-                raise TypeError(
-                    "start_soon got unexpected {!r} – are you trying to use a "
-                    "library written for asyncio/twisted/tornado or similar? "
-                    "That won't work without some sort of compatibility shim."
-                    .format(coro)
-                )
-
-            if isasyncgen(coro):
-                raise TypeError(
-                    "start_soon expected an async function but got an async "
-                    "generator {!r}".format(coro)
-                )
-
-            # Give good error for: nursery.start_soon(some_sync_fn)
-            raise TypeError(
-                "Trio expected an async function, but {!r} appears to be "
-                "synchronous".format(
-                    getattr(async_fn, "__qualname__", async_fn)
-                )
-            )
-
-        ######
-        # Set up the Task object
-        ######
+        coro = coroutine_or_error(async_fn, *args)
 
         if name is None:
             name = async_fn
@@ -1356,7 +1273,10 @@ class Runner:
             LOCALS_KEY_KI_PROTECTION_ENABLED, system_task
         )
 
-        task = Task(
+        ######
+        # Set up the Task object
+        ######
+        task = Task._create(
             coro=coro,
             parent_nursery=nursery,
             runner=self,
@@ -1492,7 +1412,7 @@ class Runner:
 
         """
         if self.trio_token is None:
-            self.trio_token = TrioToken(self.entry_queue)
+            self.trio_token = TrioToken._create(self.entry_queue)
         return self.trio_token
 
     ################
@@ -1578,18 +1498,18 @@ class Runner:
                      nursery.start_soon(lock_taker, lock)
                      # child hasn't run yet, we have the lock
                      assert lock.locked()
-                     assert lock._owner is trio.hazmat.current_task()
+                     assert lock._owner is trio.lowlevel.current_task()
                      await trio.testing.wait_all_tasks_blocked()
                      # now the child has run and is blocked on lock.acquire(), we
                      # still have the lock
                      assert lock.locked()
-                     assert lock._owner is trio.hazmat.current_task()
+                     assert lock._owner is trio.lowlevel.current_task()
                      lock.release()
                      try:
                          # The child has a prior claim, so we can't have it
                          lock.acquire_nowait()
                      except trio.WouldBlock:
-                         assert lock._owner is not trio.hazmat.current_task()
+                         assert lock._owner is not trio.lowlevel.current_task()
                          print("PASS")
                      else:
                          print("FAIL")
@@ -1909,7 +1829,7 @@ def run_impl(runner, async_fn, args):
             try:
                 # We used to unwrap the Outcome object here and send/throw its
                 # contents in directly, but it turns out that .throw() is
-                # buggy, at least on CPython 3.6 and earlier:
+                # buggy, at least on CPython 3.6:
                 #   https://bugs.python.org/issue29587
                 #   https://bugs.python.org/issue29590
                 # So now we send in the Outcome object and unwrap it on the
@@ -2048,7 +1968,7 @@ async def checkpoint_if_cancelled():
     Equivalent to (but potentially more efficient than)::
 
         if trio.current_deadline() == -inf:
-            await trio.hazmat.checkpoint()
+            await trio.lowlevel.checkpoint()
 
     This is either a no-op, or else it allow other tasks to be scheduled and
     then raises :exc:`trio.Cancelled`.
