@@ -1,10 +1,8 @@
 import attr
 
 from .. import _core
-from .._util import aiter_compat
 from .._deprecate import deprecated
-
-__all__ = ["UnboundedQueue"]
+from .._util import SubclassingDeprecatedIn_v0_15_0
 
 
 @attr.s(frozen=True)
@@ -13,7 +11,7 @@ class _UnboundedQueueStats:
     tasks_waiting = attr.ib()
 
 
-class UnboundedQueue:
+class UnboundedQueue(metaclass=SubclassingDeprecatedIn_v0_15_0):
     """An unbounded queue suitable for certain unusual forms of inter-task
     communication.
 
@@ -42,11 +40,10 @@ class UnboundedQueue:
            ...
 
     """
-
     @deprecated(
         "0.9.0",
         issue=497,
-        thing="trio.hazmat.UnboundedQueue",
+        thing="trio.lowlevel.UnboundedQueue",
         instead="trio.open_memory_channel(math.inf)"
     )
     def __init__(self):
@@ -147,7 +144,6 @@ class UnboundedQueue:
             tasks_waiting=self._lot.statistics().tasks_waiting
         )
 
-    @aiter_compat
     def __aiter__(self):
         return self
 
