@@ -176,11 +176,30 @@ async def test_agen_protection():
         finally:
             assert not _core.currently_ki_protected()
 
+    # Native async generators
+    @_core.enable_ki_protection
+    async def agen_protected3():
+        assert _core.currently_ki_protected()
+        try:
+            yield
+        finally:
+            assert _core.currently_ki_protected()
+
+    @_core.disable_ki_protection
+    async def agen_unprotected3():
+        assert not _core.currently_ki_protected()
+        try:
+            yield
+        finally:
+            assert not _core.currently_ki_protected()
+
     for agen_fn in [
         agen_protected1,
         agen_protected2,
+        agen_protected3,
         agen_unprotected1,
         agen_unprotected2,
+        agen_unprotected3,
     ]:
         async for _ in agen_fn():  # noqa
             assert not _core.currently_ki_protected()
@@ -480,10 +499,6 @@ def test_ki_with_broken_threads():
 # For details on why this test is non-trivial, see:
 #   https://github.com/python-trio/trio/issues/42
 #   https://github.com/python-trio/trio/issues/109
-# To make it an even better test, we should try doing
-#   pthread_kill(pthread_self, SIGINT)
-# in the child thread, to make sure signals in non-main threads also wake up
-# the main loop... but currently that test would fail (see gh-109 again).
 @slow
 def test_ki_wakes_us_up():
     assert is_main_thread()
@@ -508,17 +523,14 @@ def test_ki_wakes_us_up():
     #
     # which contains the desired sequence.
     #
-    # Affected version of CPython include:
-    # - all versions of 3.5 (fix will not be backported)
-    # - 3.6.1 and earlier
+    # Affected version of CPython include 3.6.1 and earlier.
     # It's fixed in 3.6.2 and 3.7+
     #
     # PyPy was never affected.
     #
-    # The problem technically occurs on Unix as well, if a signal is delivered
-    # to a non-main thread, and if we were relying on the wakeup fd to wake
-    # us. Currently we don't use the wakeup fd on Unix anyway, though (see
-    # gh-109).
+    # The problem technically can occur on Unix as well, if a signal is
+    # delivered to a non-main thread, though we haven't observed this in
+    # practice.
     #
     # There's also this theoretical problem, but hopefully it won't actually
     # bite us in practice:
@@ -526,8 +538,8 @@ def test_ki_wakes_us_up():
     #   https://bitbucket.org/pypy/pypy/issues/2623
     import platform
     buggy_wakeup_fd = (
-        os.name == "nt" and platform.python_implementation() == "CPython"
-        and sys.version_info < (3, 6, 2)
+        platform.python_implementation() == "CPython" and sys.version_info <
+        (3, 6, 2)
     )
 
     # lock is only needed to avoid an annoying race condition where the
