@@ -14,13 +14,15 @@ import attr
 import outcome
 import sniffio
 import pytest
-from async_generator import async_generator
 
-from .tutil import slow, check_sequence_matches, gc_collect_harder
+from .tutil import (
+    slow, check_sequence_matches, gc_collect_harder,
+    ignore_coroutine_never_awaited_warnings
+)
+
 from ... import _core
 from ..._threads import to_thread_run_sync
 from ..._timeouts import sleep, fail_after
-from ..._util import aiter_compat
 from ...testing import (
     wait_all_tasks_blocked,
     Sequencer,
@@ -35,26 +37,6 @@ async def sleep_forever():
     return await _core.wait_task_rescheduled(lambda _: _core.Abort.SUCCEEDED)
 
 
-# Some of our tests need to leak coroutines, and thus trigger the
-# "RuntimeWarning: coroutine '...' was never awaited" message. This context
-# manager should be used anywhere this happens to hide those messages, because
-# (a) when expected they're clutter, (b) on CPython 3.5.x where x < 3, this
-# warning can trigger a segfault if we run with warnings turned into errors:
-#   https://bugs.python.org/issue27811
-@contextmanager
-def ignore_coroutine_never_awaited_warnings():
-    with warnings.catch_warnings():
-        warnings.filterwarnings(
-            "ignore", message="coroutine '.*' was never awaited"
-        )
-        try:
-            yield
-        finally:
-            # Make sure to trigger any coroutine __del__ methods now, before
-            # we leave the context manager.
-            gc_collect_harder()
-
-
 def test_basic():
     async def trivial(x):
         return x
@@ -1380,13 +1362,7 @@ async def test_TrioToken_run_sync_soon_idempotent():
         for i in range(100):
             token.run_sync_soon(cb, i, idempotent=True)
     await wait_all_tasks_blocked()
-    if (
-        sys.version_info < (3, 6)
-        and platform.python_implementation() == "CPython"
-    ):
-        # no order guarantees
-        record.sort()
-    # Otherwise, we guarantee FIFO
+    # We guarantee FIFO
     assert record == list(range(100))
 
 
@@ -1706,8 +1682,6 @@ async def test_current_effective_deadline(mock_clock):
     assert _core.current_effective_deadline() == inf
 
 
-# @coroutine is deprecated since python 3.8, which is fine with us.
-@pytest.mark.filterwarnings("ignore:.*@coroutine.*:DeprecationWarning")
 def test_nice_error_on_bad_calls_to_run_or_spawn():
     def bad_call_run(*args):
         _core.run(*args)
@@ -1719,60 +1693,22 @@ def test_nice_error_on_bad_calls_to_run_or_spawn():
 
         _core.run(main)
 
-    class Deferred:
-        "Just kidding"
+    for bad_call in bad_call_run, bad_call_spawn:
 
-    with ignore_coroutine_never_awaited_warnings():
-        for bad_call in bad_call_run, bad_call_spawn:
+        async def f():  # pragma: no cover
+            pass
 
-            async def f():  # pragma: no cover
-                pass
+        with pytest.raises(TypeError, match="expecting an async function"):
+            bad_call(f())
 
-            with pytest.raises(TypeError) as excinfo:
-                bad_call(f())
-            assert "expecting an async function" in str(excinfo.value)
+        async def async_gen(arg):  # pragma: no cover
+            yield arg
 
-            import asyncio
-
-            @asyncio.coroutine
-            def generator_based_coro():  # pragma: no cover
-                yield from asyncio.sleep(1)
-
-            with pytest.raises(TypeError) as excinfo:
-                bad_call(generator_based_coro())
-            assert "asyncio" in str(excinfo.value)
-
-            with pytest.raises(TypeError) as excinfo:
-                bad_call(asyncio.Future())
-            assert "asyncio" in str(excinfo.value)
-
-            with pytest.raises(TypeError) as excinfo:
-                bad_call(lambda: asyncio.Future())
-            assert "asyncio" in str(excinfo.value)
-
-            with pytest.raises(TypeError) as excinfo:
-                bad_call(Deferred())
-            assert "twisted" in str(excinfo.value)
-
-            with pytest.raises(TypeError) as excinfo:
-                bad_call(lambda: Deferred())
-            assert "twisted" in str(excinfo.value)
-
-            with pytest.raises(TypeError) as excinfo:
-                bad_call(len, [1, 2, 3])
-            assert "appears to be synchronous" in str(excinfo.value)
-
-            @async_generator
-            async def async_gen(arg):  # pragma: no cover
-                pass
-
-            with pytest.raises(TypeError) as excinfo:
-                bad_call(async_gen, 0)
-            msg = "expected an async function but got an async generator"
-            assert msg in str(excinfo.value)
-
-            # Make sure no references are kept around to keep anything alive
-            del excinfo
+        with pytest.raises(
+            TypeError,
+            match="expected an async function but got an async generator"
+        ):
+            bad_call(async_gen, 0)
 
 
 def test_calling_asyncio_function_gives_nice_error():
@@ -2049,7 +1985,6 @@ async def test_nursery_stop_async_iteration():
         async def _accumulate(self, f, items, i):
             items[i] = await f()
 
-        @aiter_compat
         def __aiter__(self):
             return self
 
@@ -2167,11 +2102,7 @@ def test_system_task_contexts():
 
 
 def test_Nursery_init():
-    check_Nursery_error = pytest.raises(
-        TypeError, match='no public constructor available'
-    )
-
-    with check_Nursery_error:
+    with pytest.raises(TypeError):
         _core._run.Nursery(None, None)
 
 
@@ -2182,23 +2113,17 @@ async def test_Nursery_private_init():
 
 
 def test_Nursery_subclass():
-    with pytest.raises(
-        TypeError, match='`Nursery` does not support subclassing'
-    ):
+    with pytest.raises(TypeError):
 
         class Subclass(_core._run.Nursery):
             pass
 
 
 def test_Cancelled_init():
-    check_Cancelled_error = pytest.raises(
-        TypeError, match='no public constructor available'
-    )
-
-    with check_Cancelled_error:
+    with pytest.raises(TypeError):
         raise _core.Cancelled
 
-    with check_Cancelled_error:
+    with pytest.raises(TypeError):
         _core.Cancelled()
 
     # private constructor should not raise
@@ -2211,18 +2136,14 @@ def test_Cancelled_str():
 
 
 def test_Cancelled_subclass():
-    with pytest.raises(
-        TypeError, match='`Cancelled` does not support subclassing'
-    ):
+    with pytest.raises(TypeError):
 
         class Subclass(_core.Cancelled):
             pass
 
 
 def test_CancelScope_subclass():
-    with pytest.raises(
-        TypeError, match='`CancelScope` does not support subclassing'
-    ):
+    with pytest.raises(TypeError):
 
         class Subclass(_core.CancelScope):
             pass
@@ -2389,23 +2310,10 @@ def test_async_function_implemented_in_C():
     # These used to crash because we'd try to mutate the coroutine object's
     # cr_frame, but C functions don't have Python frames.
 
-    ns = {"_core": _core}
-    try:
-        exec(
-            dedent(
-                """
-                async def agen_fn(record):
-                    assert not _core.currently_ki_protected()
-                    record.append("the generator ran")
-                    yield
-                """
-            ),
-            ns,
-        )
-    except SyntaxError:
-        pytest.skip("Requires Python 3.6+")
-    else:
-        agen_fn = ns["agen_fn"]
+    async def agen_fn(record):
+        assert not _core.currently_ki_protected()
+        record.append("the generator ran")
+        yield
 
     run_record = []
     agen = agen_fn(run_record)
