@@ -1,3 +1,4 @@
+import os
 from contextlib import contextmanager
 
 import trio
@@ -9,8 +10,6 @@ try:
 except ImportError:
     has_unix = False
 
-__all__ = ["open_unix_socket"]
-
 
 @contextmanager
 def close_on_error(obj):
@@ -44,6 +43,6 @@ async def open_unix_socket(filename,):
     # possible location to connect to
     sock = socket(AF_UNIX, SOCK_STREAM)
     with close_on_error(sock):
-        await sock.connect(trio._util.fspath(filename))
+        await sock.connect(os.fspath(filename))
 
     return trio.SocketStream(sock)
