@@ -158,7 +158,7 @@ import trio
 from .abc import Stream, Listener
 from ._highlevel_generic import aclose_forcefully
 from . import _sync
-from ._util import ConflictDetector
+from ._util import ConflictDetector, SubclassingDeprecatedIn_v0_15_0
 from ._deprecate import warn_deprecated
 
 ################################################################
@@ -224,7 +224,7 @@ class _Once:
 _State = _Enum("_State", ["OK", "BROKEN", "CLOSED"])
 
 
-class SSLStream(Stream):
+class SSLStream(Stream, metaclass=SubclassingDeprecatedIn_v0_15_0):
     r"""Encrypted communication using SSL/TLS.
 
     :class:`SSLStream` wraps an arbitrary :class:`~trio.abc.Stream`, and
@@ -338,6 +338,7 @@ class SSLStream(Stream):
             )
         self._https_compatible = https_compatible
         self._outgoing = _stdlib_ssl.MemoryBIO()
+        self._delayed_outgoing = None
         self._incoming = _stdlib_ssl.MemoryBIO()
         self._ssl_object = ssl_context.wrap_bio(
             self._incoming,
@@ -428,8 +429,10 @@ class SSLStream(Stream):
     # comments, though, just make sure to think carefully if you ever have to
     # touch it. The big comment at the top of this file will help explain
     # too.
-    async def _retry(self, fn, *args, ignore_want_read=False):
-        await trio.hazmat.checkpoint_if_cancelled()
+    async def _retry(
+        self, fn, *args, ignore_want_read=False, is_handshake=False
+    ):
+        await trio.lowlevel.checkpoint_if_cancelled()
         yielded = False
         finished = False
         while not finished:
@@ -473,6 +476,32 @@ class SSLStream(Stream):
                 finished = True
             to_send = self._outgoing.read()
 
+            # Some versions of SSL_do_handshake have a bug in how they handle
+            # the TLS 1.3 handshake on the server side: after the handshake
+            # finishes, they automatically send session tickets, even though
+            # the client may not be expecting data to arrive at this point and
+            # sending it could cause a deadlock or lost data. This applies at
+            # least to OpenSSL 1.1.1c and earlier, and the OpenSSL devs
+            # currently have no plans to fix it:
+            #
+            #   https://github.com/openssl/openssl/issues/7948
+            #   https://github.com/openssl/openssl/issues/7967
+            #
+            # The correct behavior is to wait to send session tickets on the
+            # first call to SSL_write. (This is what BoringSSL does.) So, we
+            # use a heuristic to detect when OpenSSL has tried to send session
+            # tickets, and we manually delay sending them until the
+            # appropriate moment. For more discussion see:
+            #
+            #   https://github.com/python-trio/trio/issues/819#issuecomment-517529763
+            if (
+                is_handshake and not want_read and self._ssl_object.server_side
+                and self._ssl_object.version() == "TLSv1.3"
+            ):
+                assert self._delayed_outgoing is None
+                self._delayed_outgoing = to_send
+                to_send = b""
+
             # Outputs from the above code block are:
             #
             # - to_send: bytestring; if non-empty then we need to send
@@ -535,6 +564,9 @@ class SSLStream(Stream):
                 async with self._inner_send_lock:
                     yielded = True
                     try:
+                        if self._delayed_outgoing is not None:
+                            to_send = self._delayed_outgoing + to_send
+                            self._delayed_outgoing = None
                         await self.transport_stream.send_all(to_send)
                     except:
                         # Some unknown amount of our data got sent, and we
@@ -566,12 +598,12 @@ class SSLStream(Stream):
                             self._incoming.write(data)
                         self._inner_recv_count += 1
         if not yielded:
-            await trio.hazmat.cancel_shielded_checkpoint()
+            await trio.lowlevel.cancel_shielded_checkpoint()
         return ret
 
     async def _do_handshake(self):
         try:
-            await self._retry(self._ssl_object.do_handshake)
+            await self._retry(self._ssl_object.do_handshake, is_handshake=True)
         except:
             self._state = _State.BROKEN
             raise
@@ -638,7 +670,7 @@ class SSLStream(Stream):
                         (_stdlib_ssl.SSLEOFError, _stdlib_ssl.SSLSyscallError)
                     )
                 ):
-                    await trio.hazmat.checkpoint()
+                    await trio.lowlevel.checkpoint()
                     return b""
                 else:
                     raise
@@ -665,7 +697,7 @@ class SSLStream(Stream):
                     self._https_compatible
                     and isinstance(exc.__cause__, _stdlib_ssl.SSLEOFError)
                 ):
-                    await trio.hazmat.checkpoint()
+                    await trio.lowlevel.checkpoint()
                     return b""
                 else:
                     raise
@@ -687,7 +719,7 @@ class SSLStream(Stream):
             # SSLObject interprets write(b"") as an EOF for some reason, which
             # is not what we want.
             if not data:
-                await trio.hazmat.checkpoint()
+                await trio.lowlevel.checkpoint()
                 return
             await self._retry(self._ssl_object.write, data)
 
@@ -731,7 +763,7 @@ class SSLStream(Stream):
 
         """
         if self._state is _State.CLOSED:
-            await trio.hazmat.checkpoint()
+            await trio.lowlevel.checkpoint()
             return
         if self._state is _State.BROKEN or self._https_compatible:
             self._state = _State.CLOSED
@@ -850,7 +882,9 @@ class SSLStream(Stream):
                 await self.transport_stream.wait_send_all_might_not_block()
 
 
-class SSLListener(Listener[SSLStream]):
+class SSLListener(
+    Listener[SSLStream], metaclass=SubclassingDeprecatedIn_v0_15_0
+):
     """A :class:`~trio.abc.Listener` for SSL/TLS-encrypted servers.
 
     :class:`SSLListener` wraps around another Listener, and converts
@@ -871,7 +905,6 @@ class SSLListener(Listener[SSLStream]):
           passed to ``__init__``.
 
     """
-
     def __init__(
         self,
         transport_listener,
