@@ -3,12 +3,14 @@ import queue as stdlib_queue
 from itertools import count
 
 import attr
+import inspect
 import outcome
 
 import trio
 
 from ._sync import CapacityLimiter
 from ._core import enable_ki_protection, disable_ki_protection, RunVar, TrioToken
+from ._util import coroutine_or_error
 
 # Global due to Threading API, thread local storage for trio token
 TOKEN_LOCAL = threading.local()
@@ -17,7 +19,7 @@ TOKEN_LOCAL = threading.local()
 class BlockingTrioPortal:
     def __init__(self, trio_token=None):
         if trio_token is None:
-            trio_token = trio.hazmat.current_trio_token()
+            trio_token = trio.lowlevel.current_trio_token()
         self._trio_token = trio_token
 
     def run(self, afn, *args):
@@ -238,15 +240,14 @@ async def to_thread_run_sync(sync_fn, *args, cancellable=False, limiter=None):
       Exception: Whatever ``sync_fn(*args)`` raises.
 
     """
-    await trio.hazmat.checkpoint_if_cancelled()
-    token = trio.hazmat.current_trio_token()
+    await trio.lowlevel.checkpoint_if_cancelled()
     if limiter is None:
         limiter = current_default_thread_limiter()
 
     # Holds a reference to the task that's blocked in this function waiting
     # for the result – or None if this function was cancelled and we should
     # discard the result.
-    task_register = [trio.hazmat.current_task()]
+    task_register = [trio.lowlevel.current_task()]
     name = "trio-worker-{}".format(next(_thread_counter))
     placeholder = ThreadPlaceholder(name)
 
@@ -265,7 +266,7 @@ async def to_thread_run_sync(sync_fn, *args, cancellable=False, limiter=None):
 
         result = outcome.capture(do_release_then_return_result)
         if task_register[0] is not None:
-            trio.hazmat.reschedule(task_register[0], result)
+            trio.lowlevel.reschedule(task_register[0], result)
 
     # This is the function that runs in the worker thread to do the actual
     # work and then schedule the call to report_back_in_trio_thread_fn
@@ -276,7 +277,7 @@ async def to_thread_run_sync(sync_fn, *args, cancellable=False, limiter=None):
         try:
             result = outcome.capture(sync_fn, *args)
             try:
-                token.run_sync_soon(report_back_in_trio_thread_fn, result)
+                trio_token.run_sync_soon(report_back_in_trio_thread_fn, result)
             except trio.RunFinishedError:
                 # The entire run finished, so our particular task is certainly
                 # long gone -- it must have cancelled.
@@ -288,7 +289,7 @@ async def to_thread_run_sync(sync_fn, *args, cancellable=False, limiter=None):
     try:
         # daemon=True because it might get left behind if we cancel, and in
         # this case shouldn't block process exit.
-        current_trio_token = trio.hazmat.current_trio_token()
+        current_trio_token = trio.lowlevel.current_trio_token()
         thread = threading.Thread(
             target=worker_thread_fn,
             args=(current_trio_token,),
@@ -303,11 +304,11 @@ async def to_thread_run_sync(sync_fn, *args, cancellable=False, limiter=None):
     def abort(_):
         if cancellable:
             task_register[0] = None
-            return trio.hazmat.Abort.SUCCEEDED
+            return trio.lowlevel.Abort.SUCCEEDED
         else:
-            return trio.hazmat.Abort.FAILED
+            return trio.lowlevel.Abort.FAILED
 
-    return await trio.hazmat.wait_task_rescheduled(abort)
+    return await trio.lowlevel.wait_task_rescheduled(abort)
 
 
 def _run_fn_as_system_task(cb, fn, *args, trio_token=None):
@@ -333,7 +334,7 @@ def _run_fn_as_system_task(cb, fn, *args, trio_token=None):
     # thread local storage (or the absence of) is sufficient to check if trio
     # is running in a thread or not.
     try:
-        trio.hazmat.current_task()
+        trio.lowlevel.current_task()
     except RuntimeError:
         pass
     else:
@@ -366,6 +367,7 @@ def from_thread_run(afn, *args, trio_token=None):
             which would otherwise cause a deadlock.
         AttributeError: if no ``trio_token`` was provided, and we can't infer
             one from context.
+        TypeError: if ``afn`` is not an asynchronous function.
 
     **Locating a Trio Token**: There are two ways to specify which
     `trio.run` loop to reenter:
@@ -381,12 +383,13 @@ def from_thread_run(afn, *args, trio_token=None):
     def callback(q, afn, args):
         @disable_ki_protection
         async def unprotected_afn():
-            return await afn(*args)
+            coro = coroutine_or_error(afn, *args)
+            return await coro
 
         async def await_in_trio_thread_task():
             q.put_nowait(await outcome.acapture(unprotected_afn))
 
-        trio.hazmat.spawn_system_task(await_in_trio_thread_task, name=afn)
+        trio.lowlevel.spawn_system_task(await_in_trio_thread_task, name=afn)
 
     return _run_fn_as_system_task(callback, afn, *args, trio_token=trio_token)
 
@@ -404,13 +407,11 @@ def from_thread_run_sync(fn, *args, trio_token=None):
     Raises:
         RunFinishedError: if the corresponding call to `trio.run` has
             already completed.
-        Cancelled: if the corresponding call to `trio.run` completes
-            while ``afn(*args)`` is running, then ``afn`` is likely to raise
-            :exc:`trio.Cancelled`, and this will propagate out into
         RuntimeError: if you try calling this from inside the Trio thread,
             which would otherwise cause a deadlock.
         AttributeError: if no ``trio_token`` was provided, and we can't infer
             one from context.
+        TypeError: if ``fn`` is an async function.
 
     **Locating a Trio Token**: There are two ways to specify which
     `trio.run` loop to reenter:
@@ -426,7 +427,17 @@ def from_thread_run_sync(fn, *args, trio_token=None):
     def callback(q, fn, args):
         @disable_ki_protection
         def unprotected_fn():
-            return fn(*args)
+            ret = fn(*args)
+
+            if inspect.iscoroutine(ret):
+                # Manually close coroutine to avoid RuntimeWarnings
+                ret.close()
+                raise TypeError(
+                    "Trio expected a sync function, but {!r} appears to be "
+                    "asynchronous".format(getattr(fn, "__qualname__", fn))
+                )
+
+            return ret
 
         res = outcome.capture(unprotected_fn)
         q.put_nowait(res)
