@@ -8,14 +8,11 @@ import pathlib
 from functools import wraps, update_wrapper
 import typing as t
 import threading
+import collections
 
-import async_generator
+from async_generator import isasyncgen
 
-# There's a dependency loop here... _core is allowed to use this file (in fact
-# it's the *only* file in the main trio/ package it's allowed to use), but
-# ConflictDetector needs checkpoint so it also has to import
-# _core. Possibly we should split this file into two: one for true generic
-# low-level utility code, and one for higher level helpers?
+from ._deprecate import warn_deprecated
 
 import trio
 
@@ -68,24 +65,6 @@ else:
         signal.pthread_kill(threading.get_ident(), signum)
 
 
-# Decorator to handle the change to __aiter__ in 3.5.2
-if sys.version_info < (3, 5, 2):
-
-    def aiter_compat(aiter_impl):
-        # de-sugar decorator to fix Python 3.8 coverage issue
-        # https://github.com/python-trio/trio/pull/784#issuecomment-446438407
-        async def __aiter__(*args, **kwargs):
-            return aiter_impl(*args, **kwargs)
-
-        __aiter__ = wraps(aiter_impl)(__aiter__)
-
-        return __aiter__
-else:
-
-    def aiter_compat(aiter_impl):
-        return aiter_impl
-
-
 # See: #461 as to why this is needed.
 # The gist is that threading.main_thread() has the capability to lie to us
 # if somebody else edits the threading ident cache to replace the main
@@ -103,6 +82,91 @@ def is_main_thread():
         return False
 
 
+######
+# Call the function and get the coroutine object, while giving helpful
+# errors for common mistakes. Returns coroutine object.
+######
+def coroutine_or_error(async_fn, *args):
+    def _return_value_looks_like_wrong_library(value):
+        # Returned by legacy @asyncio.coroutine functions, which includes
+        # a surprising proportion of asyncio builtins.
+        if isinstance(value, collections.abc.Generator):
+            return True
+        # The protocol for detecting an asyncio Future-like object
+        if getattr(value, "_asyncio_future_blocking", None) is not None:
+            return True
+        # This janky check catches tornado Futures and twisted Deferreds.
+        # By the time we're calling this function, we already know
+        # something has gone wrong, so a heuristic is pretty safe.
+        if value.__class__.__name__ in ("Future", "Deferred"):
+            return True
+        return False
+
+    try:
+        coro = async_fn(*args)
+
+    except TypeError:
+        # Give good error for: nursery.start_soon(trio.sleep(1))
+        if isinstance(async_fn, collections.abc.Coroutine):
+            # explicitly close coroutine to avoid RuntimeWarning
+            async_fn.close()
+
+            raise TypeError(
+                "Trio was expecting an async function, but instead it got "
+                "a coroutine object {async_fn!r}\n"
+                "\n"
+                "Probably you did something like:\n"
+                "\n"
+                "  trio.run({async_fn.__name__}(...))            # incorrect!\n"
+                "  nursery.start_soon({async_fn.__name__}(...))  # incorrect!\n"
+                "\n"
+                "Instead, you want (notice the parentheses!):\n"
+                "\n"
+                "  trio.run({async_fn.__name__}, ...)            # correct!\n"
+                "  nursery.start_soon({async_fn.__name__}, ...)  # correct!"
+                .format(async_fn=async_fn)
+            ) from None
+
+        # Give good error for: nursery.start_soon(future)
+        if _return_value_looks_like_wrong_library(async_fn):
+            raise TypeError(
+                "Trio was expecting an async function, but instead it got "
+                "{!r} – are you trying to use a library written for "
+                "asyncio/twisted/tornado or similar? That won't work "
+                "without some sort of compatibility shim.".format(async_fn)
+            ) from None
+
+        raise
+
+    # We can't check iscoroutinefunction(async_fn), because that will fail
+    # for things like functools.partial objects wrapping an async
+    # function. So we have to just call it and then check whether the
+    # return value is a coroutine object.
+    if not isinstance(coro, collections.abc.Coroutine):
+        # Give good error for: nursery.start_soon(func_returning_future)
+        if _return_value_looks_like_wrong_library(coro):
+            raise TypeError(
+                "Trio got unexpected {!r} – are you trying to use a "
+                "library written for asyncio/twisted/tornado or similar? "
+                "That won't work without some sort of compatibility shim."
+                .format(coro)
+            )
+
+        if isasyncgen(coro):
+            raise TypeError(
+                "start_soon expected an async function but got an async "
+                "generator {!r}".format(coro)
+            )
+
+        # Give good error for: nursery.start_soon(some_sync_fn)
+        raise TypeError(
+            "Trio expected an async function, but {!r} appears to be "
+            "synchronous".format(getattr(async_fn, "__qualname__", async_fn))
+        )
+
+    return coro
+
+
 class ConflictDetector:
     """Detect when two tasks are about to perform operations that would
     conflict.
@@ -177,66 +241,6 @@ def fixup_module_metadata(module_name, namespace):
             fix_one(objname, objname, obj)
 
 
-# os.fspath is defined on Python 3.6+ but we need to support Python 3.5 too
-# This is why we provide our own implementation. On Python 3.6+ we use the
-# StdLib's version and on Python 3.5 our own version.
-# Our own implementation implementation is based on PEP 519 while it has also
-# been adapted to work with pathlib objects on python 3.5
-# The input typehint is removed as there is no os.PathLike on 3.5.
-# See: https://www.python.org/dev/peps/pep-0519/#os
-
-
-def fspath(path) -> t.Union[str, bytes]:
-    """Return the path representation of a path-like object.
-
-    Returns
-    -------
-    - If str or bytes is passed in, it is returned unchanged.
-    - If the os.PathLike interface is implemented it is used to get the path
-      representation.
-    - If the python version is 3.5 or earlier and a pathlib object is passed,
-      the object's string representation is returned.
-
-    Raises
-    ------
-    - Regardless of the input, if the path representation (e.g. the value
-      returned from __fspath__) is not str or bytes, TypeError is raised.
-    - If the provided path is not str, bytes, pathlib.PurePath or os.PathLike,
-      TypeError is raised.
-    """
-    if isinstance(path, (str, bytes)):
-        return path
-    # Work from the object's type to match method resolution of other magic
-    # methods.
-    path_type = type(path)
-    # On python 3.5, pathlib objects don't have the __fspath__ method,
-    # but we still want to get their string representation.
-    if issubclass(path_type, pathlib.PurePath):
-        return str(path)
-    try:
-        path_repr = path_type.__fspath__(path)
-    except AttributeError:
-        if hasattr(path_type, '__fspath__'):
-            raise
-        else:
-            raise TypeError(
-                "expected str, bytes or os.PathLike object, "
-                "not " + path_type.__name__
-            )
-    if isinstance(path_repr, (str, bytes)):
-        return path_repr
-    else:
-        raise TypeError(
-            "expected {}.__fspath__() to return str or bytes, "
-            "not {}".format(path_type.__name__,
-                            type(path_repr).__name__)
-        )
-
-
-if hasattr(os, "fspath"):
-    fspath = os.fspath  # noqa
-
-
 class generic_function:
     """Decorator that makes a function indexable, to communicate
     non-inferrable generic type parameters to a static type checker.
@@ -266,7 +270,7 @@ class generic_function:
 
 # If a new class inherits from any ABC, then the new class's metaclass has to
 # inherit from ABCMeta. If a new class inherits from typing.Generic, and
-# you're using Python 3.6 or earlier, then the new class's metaclass has to
+# you're using Python 3.6, then the new class's metaclass has to
 # inherit from typing.GenericMeta. Some of the classes that want to use Final
 # or NoPublicConstructor inherit from ABCs and generics, so Final has to
 # inherit from these metaclasses. Fortunately, GenericMeta inherits from
@@ -296,11 +300,25 @@ class Final(BaseMeta):
         for base in bases:
             if isinstance(base, Final):
                 raise TypeError(
-                    "`%s` does not support subclassing" % base.__name__
+                    f"{base.__module__}.{base.__qualname__} does not support subclassing"
                 )
         return super().__new__(cls, name, bases, cls_namespace)
 
 
+class SubclassingDeprecatedIn_v0_15_0(BaseMeta):
+    def __new__(cls, name, bases, cls_namespace):
+        for base in bases:
+            if isinstance(base, SubclassingDeprecatedIn_v0_15_0):
+                warn_deprecated(
+                    f"subclassing {base.__module__}.{base.__qualname__}",
+                    "0.15.0",
+                    issue=1044,
+                    instead="composition or delegation"
+                )
+                break
+        return super().__new__(cls, name, bases, cls_namespace)
+
+
 class NoPublicConstructor(Final):
     """Metaclass that enforces a class to be final (i.e., subclass not allowed)
     and ensures a private constructor.
@@ -320,7 +338,9 @@ class NoPublicConstructor(Final):
     - TypeError if a sub class or an instance is created.
     """
     def __call__(self, *args, **kwargs):
-        raise TypeError("no public constructor available")
+        raise TypeError(
+            f"{self.__module__}.{self.__qualname__} has no public constructor"
+        )
 
     def _create(self, *args, **kwargs):
         return super().__call__(*args, **kwargs)
