@@ -5,16 +5,6 @@ from .._highlevel_generic import StapledStream
 from .. import _util
 from ..abc import SendStream, ReceiveStream
 
-__all__ = [
-    "MemorySendStream",
-    "MemoryReceiveStream",
-    "memory_stream_pump",
-    "memory_stream_one_way_pair",
-    "memory_stream_pair",
-    "lockstep_stream_one_way_pair",
-    "lockstep_stream_pair",
-]
-
 ################################################################
 # In-memory streams - Unbounded buffer version
 ################################################################
@@ -83,7 +73,9 @@ class _UnboundedByteQueue:
             return self._get_impl(max_bytes)
 
 
-class MemorySendStream(SendStream):
+class MemorySendStream(
+    SendStream, metaclass=_util.SubclassingDeprecatedIn_v0_15_0
+):
     """An in-memory :class:`~trio.abc.SendStream`.
 
     Args:
@@ -103,7 +95,6 @@ class MemorySendStream(SendStream):
        you can change them at any time.
 
     """
-
     def __init__(
         self,
         send_all_hook=None,
@@ -199,7 +190,9 @@ class MemorySendStream(SendStream):
         return self._outgoing.get_nowait(max_bytes)
 
 
-class MemoryReceiveStream(ReceiveStream):
+class MemoryReceiveStream(
+    ReceiveStream, metaclass=_util.SubclassingDeprecatedIn_v0_15_0
+):
     """An in-memory :class:`~trio.abc.ReceiveStream`.
 
     Args:
@@ -215,7 +208,6 @@ class MemoryReceiveStream(ReceiveStream):
        change them at any time.
 
     """
-
     def __init__(self, receive_some_hook=None, close_hook=None):
         self._conflict_detector = _util.ConflictDetector(
             "another task is using this stream"
@@ -395,7 +387,7 @@ def memory_stream_pair():
         async def trickle():
             # left is a StapledStream, and left.send_stream is a MemorySendStream
             # right is a StapledStream, and right.recv_stream is a MemoryReceiveStream
-            while memory_stream_pump(left.send_stream, right.recv_stream, max_byes=1):
+            while memory_stream_pump(left.send_stream, right.recv_stream, max_bytes=1):
                 # Pause between each byte
                 await trio.sleep(1)
         # Normally this send_all_hook calls memory_stream_pump directly without
