@@ -23,7 +23,7 @@ class MemoryListener(trio.abc.Listener):
         return client
 
     async def accept(self):
-        await trio.hazmat.checkpoint()
+        await trio.lowlevel.checkpoint()
         assert not self.closed
         if self.accept_hook is not None:
             await self.accept_hook()
@@ -33,7 +33,7 @@ class MemoryListener(trio.abc.Listener):
 
     async def aclose(self):
         self.closed = True
-        await trio.hazmat.checkpoint()
+        await trio.lowlevel.checkpoint()
 
 
 async def test_serve_listeners_basic():
